/*
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import XCTest
@testable import High_Roller

class WebSourceTests: XCTestCase {

  func testDownloadingPageUsingExpectation() {
    let expect = expectation(description: "waitForWebSource")
    var diceRollsReceived = 0

    let webSource = WebSource()
    webSource.findRollOnline(numberOfDice: 2) { (result) in
      diceRollsReceived = result.count
      expect.fulfill()
    }

    waitForExpectations(timeout: 10, handler: nil)
    XCTAssertEqual(diceRollsReceived, 2)
  }

  func testUsingMockURLSession() {
    let address = "https://www.random.org/dice/?num=2"
    guard let url = URL(string: address) else {
      XCTFail()
      return
    }
    let request = URLRequest(url: url)

    let mockSession = MockURLSession()
    XCTAssertFalse(mockSession.dataTask.resumeGotCalled)
    XCTAssertNil(mockSession.url)

    let task = mockSession.dataTask(with: request) { (data, response, error) in }
    task.resume()

    XCTAssertTrue(mockSession.dataTask.resumeGotCalled)
    XCTAssertEqual(mockSession.url, url)
  }

  func testParsingGoodData() {
    let webSource = WebSource()
    let goodDataString = "<p>You rolled 2 dice:</p>\n<p>\n<img src=\"dice6.png\" alt=\"6\" />\n<img src=\"dice1.png\" alt=\"1\" />\n</p>"
    guard let goodData = goodDataString.data(using: .utf8) else {
      XCTFail()
      return
    }

    let diceArray = webSource.parseIncomingData(data: goodData)
    XCTAssertEqual(diceArray, [6, 1])
  }

  func testParsingBadData() {
    let webSource = WebSource()
    let badDataString = "This string is not the expected result"
    guard let badData = badDataString.data(using: .utf8) else {
      XCTFail()
      return
    }

    let diceArray = webSource.parseIncomingData(data: badData)
    XCTAssertEqual(diceArray, [])
  }

}


class MockURLSession: URLSession {
  var url: URL?
  var dataTask = MockURLSessionTask()

  override func dataTask(with request: URLRequest, completionHandler: @escaping (Data?, URLResponse?, Error?) -> Void) -> MockURLSessionTask {
    self.url = request.url
    return dataTask
  }
}

class MockURLSessionTask: URLSessionDataTask {
  var resumeGotCalled = false

  override func resume() {
    resumeGotCalled = true
  }
}
